//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "FormJsonEditorTextView.h"
#include "Log.h"
#include <Scintilla.h> 
#include <SciLexer.h>
#include <math.h>
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TfrmJsonEditorTextView *frmJsonEditorTextView;

#define SCINT_ID 1010
static const int MARGIN_SCRIPT_FOLD_INDEX = 1;

class TScEdit : public TWinControl
{ 
protected: 
    virtual void __fastcall CreateParams(Controls::TCreateParams &Params) 
    { 
        TWinControl::CreateParams(Params); 
        CreateSubClass(Params, "Scintilla"); 
    } 
    virtual void __fastcall WndProc(Messages::TMessage &Message) 
    { 
        //LOG("Msg = %d", Message.Msg);
		TWinControl::WndProc(Message);
		if(Message.Msg == WM_GETDLGCODE)
		{
			Message.Result = DLGC_WANTALLKEYS|DLGC_WANTARROWS|DLGC_WANTTAB;
		}
    } 
public:
	__fastcall TScEdit(Classes::TComponent* AOwner)
		:TWinControl(AOwner){;}

	sptr_t SendEditor(unsigned int iMessage, uptr_t wParam = 0, sptr_t lParam = 0)
	{
		return SendMessage(Handle, iMessage, wParam, lParam);
	}

	void setCppStyle(AnsiString fontName, int fontSize)
	{
		const char* szKeywords1=
			"true false null";
		//const char* szKeywords2=
		//	"bool char float int long short void wchar_t";

		SendEditor(SCI_STYLESETFONT, STYLE_DEFAULT, (sptr_t)fontName.c_str());
		SendEditor(SCI_STYLESETSIZE, STYLE_DEFAULT, fontSize);
		SendEditor(SCI_STYLECLEARALL);

		SendEditor(SCI_SETLEXER, SCLEX_CPP);
		SendEditor(SCI_SETKEYWORDS, 0, (sptr_t)szKeywords1);
		//SendEditor(SCI_SETKEYWORDS, 1, (sptr_t)szKeywords2);

		SendEditor(SCI_STYLESETFORE, SCE_C_WORD, 0x00FF0000);
		SendEditor(SCI_STYLESETFORE, SCE_C_WORD2, 0x00800080);
		SendEditor(SCI_STYLESETBOLD, SCE_C_WORD2, TRUE);
		SendEditor(SCI_STYLESETFORE, SCE_C_STRING, 0x001515A3);
		SendEditor(SCI_STYLESETFORE, SCE_C_CHARACTER, 0x001515A3);
		SendEditor(SCI_STYLESETFORE, SCE_C_PREPROCESSOR, 0x00808080);
		SendEditor(SCI_STYLESETFORE, SCE_C_COMMENT, 0x00008000);
		SendEditor(SCI_STYLESETFORE, SCE_C_COMMENTLINE, 0x00008000);
		SendEditor(SCI_STYLESETFORE, SCE_C_COMMENTDOC, 0x00008000);
		SendEditor(SCI_STYLESETBACK, STYLE_BRACELIGHT, 0x00777777);
		SendEditor(SCI_STYLESETFORE, STYLE_BRACELIGHT, 0x00FFFFFF);
		SendEditor(SCI_STYLESETBACK, STYLE_BRACEBAD, 0x000000FF);

		SendEditor(SCI_SETCARETLINEVISIBLE, TRUE);
		SendEditor(SCI_SETCARETLINEBACK, 0xb0ffff);
	}

	void SetExtraStyle(void)
	{
		SendEditor(SCI_SETPROPERTY, (WPARAM)"fold", (LPARAM)"1");
		SendEditor(SCI_SETPROPERTY, (WPARAM)"fold.compact", (LPARAM)"0");
		SendEditor(SCI_SETPROPERTY, (WPARAM)"fold.comment", (LPARAM)"1");
		SendEditor(SCI_SETPROPERTY, (WPARAM)"fold.preprocessor", (LPARAM)"1");

		//Now resize all the margins to zero
		//(This will be done in a RecalcLineMargin method...)
		SendEditor(SCI_SETMARGINWIDTHN, MARGIN_SCRIPT_FOLD_INDEX, 0);

		//Then set the margin type and margin mask and resize it...
		SendEditor(SCI_SETMARGINTYPEN,  MARGIN_SCRIPT_FOLD_INDEX, SC_MARGIN_SYMBOL);
		SendEditor(SCI_SETMARGINMASKN, MARGIN_SCRIPT_FOLD_INDEX, SC_MASK_FOLDERS);
		SendEditor(SCI_SETMARGINWIDTHN, MARGIN_SCRIPT_FOLD_INDEX, 20);

		//Set some visual preferences
		SendEditor(SCI_MARKERDEFINE, SC_MARKNUM_FOLDER, SC_MARK_PLUS);
		SendEditor(SCI_MARKERDEFINE, SC_MARKNUM_FOLDEROPEN, SC_MARK_MINUS);
		SendEditor(SCI_MARKERDEFINE, SC_MARKNUM_FOLDEREND, SC_MARK_EMPTY);
		SendEditor(SCI_MARKERDEFINE, SC_MARKNUM_FOLDERMIDTAIL, SC_MARK_EMPTY);
		SendEditor(SCI_MARKERDEFINE, SC_MARKNUM_FOLDEROPENMID, SC_MARK_EMPTY);
		SendEditor(SCI_MARKERDEFINE, SC_MARKNUM_FOLDERSUB, SC_MARK_EMPTY);
		SendEditor(SCI_MARKERDEFINE, SC_MARKNUM_FOLDERTAIL, SC_MARK_EMPTY);

		SendEditor(SCI_SETFOLDFLAGS, 16, 0); // 16  	Draw line below if not expanded

		//Tell scintilla we want to be notified about mouse clicks in the margin
		SendEditor(SCI_SETMARGINSENSITIVEN, MARGIN_SCRIPT_FOLD_INDEX, 1);

		SendEditor(SCI_SETTABWIDTH, 4);

		SendEditor(SCI_SETMODEVENTMASK, SC_MOD_INSERTTEXT | SC_MOD_DELETETEXT, 0);

		// Add line numbers
		SendEditor(SCI_SETMARGINTYPEN, 0, SC_MARGIN_NUMBER);
		char *text = "00";
		int width = SendEditor(SCI_TEXTWIDTH, STYLE_LINENUMBER, (WPARAM)text);
		SendEditor(SCI_SETMARGINWIDTHN, 0, width+4);

		// Wider (2 pixels) caret
		SendEditor(SCI_SETCARETWIDTH, 2, 0);						
	}
};

//---------------------------------------------------------------------------
__fastcall TfrmJsonEditorTextView::TfrmJsonEditorTextView(TComponent* Owner)
	: TForm(Owner),
	callbackTextChange(NULL),
	m_se(NULL),
	lineCount(10),
	fontName("Courier New"),
	fontSize(10)	
{
	static bool once = false;
	if (!once)
	{
		once = true;
		if ( !Scintilla_RegisterClasses( GetModuleHandle(NULL) ) )
		{
			ShowMessage( "Failed to register Scintilla class" );
			return;
		}
	}
	m_se = new TScEdit (this);
	m_se-> Parent = this;
	m_se-> Align = alClient;
	m_se->setCppStyle(fontName, fontSize);
	m_se->SetExtraStyle();	
}
//---------------------------------------------------------------------------


void TfrmJsonEditorTextView::SetText(AnsiString text)
{
	m_se->SendEditor(SCI_SETTEXT, 0, (WPARAM)text.c_str());
	m_se->SendEditor(SCI_EMPTYUNDOBUFFER, 0, 0);	
}

AnsiString TfrmJsonEditorTextView::GetText(void)
{
	AnsiString ret;
	unsigned int nLength = m_se->SendEditor(SCI_GETLENGTH, 0, 0);
	ret.SetLength(nLength+2);
	m_se->SendEditor(SCI_GETTEXT, nLength+1, (WPARAM)ret.data());
	return ret;
}

void TfrmJsonEditorTextView::MarkError(int line)
{
	m_se->SendEditor(SCI_GOTOLINE, line, 0);
	unsigned int pos = m_se->SendEditor(SCI_GETCURRENTPOS, 0, 0);
	m_se->SendEditor(SCI_SETSELECTION, pos, pos+1);
}


__fastcall TfrmJsonEditorTextView::~TfrmJsonEditorTextView()
{
	delete m_se;
}

void __fastcall TfrmJsonEditorTextView::WMNotify(TMessage &Message)
{
	LPNMHDR lpnmh = reinterpret_cast<LPNMHDR>(Message.LParam);
	if (lpnmh->hwndFrom == m_se->Handle)
    {
        switch (lpnmh->code)
        {
        case SCN_MARGINCLICK:
        {
            SCNotification* notify = (SCNotification*)Message.LParam;

            //const int modifiers = notify->modifiers;
            const int position = notify->position;
            const int margin = notify->margin;
            const int line_number = m_se->SendEditor(SCI_LINEFROMPOSITION, position, 0);

            switch (margin)
            {
            case MARGIN_SCRIPT_FOLD_INDEX:
            {
                m_se->SendEditor(SCI_TOGGLEFOLD, line_number, 0);
            }
            break;
            }
		}
		break;

		case SCN_MODIFIED:	//SCEN_CHANGE:
		{
			if (callbackTextChange)
				callbackTextChange();

			int lines = m_se->SendEditor(SCI_GETLINECOUNT, 0, 0);
			if (lines < 10)
				lines = 10;
			if (lines/lineCount >= 10 || lineCount/lines >= 10)
			{
				lineCount = lines;
				int n = (int)log10(lineCount);
				char *text;
				switch (n)
				{
				case 0:
				case 1:
					text = "00";
					break;
				case 2:
					text = "000";
					break;
				case 3:
					text = "0000";
					break;
				case 4:
					text = "00000";
					break;
				default:
					text = "000000";
					break;
				}

				int width = m_se->SendEditor(SCI_TEXTWIDTH, STYLE_LINENUMBER, (WPARAM)text);
				m_se->SendEditor(SCI_SETMARGINWIDTHN, 0, width + 4);
			}
		}
		break;

		case SCN_UPDATEUI:
		{
		#if 0
			if (braceAtCaret >= 0 && braceOpposite < 0)
			{
				SendScintilla(SCI_BRACEBADLIGHT,braceAtCaret);
						SendScintilla(SCI_SETHIGHLIGHTGUIDE,0UL);
			}
		#endif
			// do brace matching
			int lStart = m_se->SendEditor(SCI_GETCURRENTPOS, 0, 0);
			int lEnd = m_se->SendEditor(SCI_BRACEMATCH, lStart-1, 0);
			// if there is a matching brace highlight it
			if (lEnd >= 0)
				m_se->SendEditor(SCI_BRACEHIGHLIGHT, lStart-1, lEnd);
			// if there is NO matching brace erase old highlight
			else
				m_se->SendEditor(SCI_BRACEHIGHLIGHT, -1, -1);
		}
		break;


		default:
		break;
        }
	}

    TForm::Dispatch(&Message);
#if 0
SCN_STYLENEEDED:
    doSciStyleNeeded(scn^.position);
SCN_CHARADDED:
    doSciCharAdded(scn^.ch);
SCN_SAVEPOINTREACHED:
    doSciSavePointReached;
SCN_SAVEPOINTLEFT:
    doSciSavePointLeft;
SCN_MODIFYATTEMPTRO:
    doSciModifyAttemptRO;
SCN_DOUBLECLICK:
    doSciDoubleClick;
SCN_UPDATEUI:
    doSciUpdateUI;
SCN_MODIFIED:
    doSciModified(scn^.position, scn^.modificationType, scn^.text, scn^.length, scn^.linesAdded, scn^.line, scn^.foldLevelNow, scn^.foldLevelPrev);
SCN_MACRORECORD:
    doSciMacroRecord(scn^.message, scn^.wParam, scn^.lParam);
SCN_MARGINCLICK:
    doSciMarginClick(scn^.modifiers, scn^.position, scn^.margin);
SCN_NEEDSHOWN:
    doSciNeedShown(scn^.position, scn^.length);
SCN_PAINTED:
    doSciPainted;
SCN_USERLISTSELECTION:
    doSciUserListSelection(scn^.listType, scn^.text);
SCN_DWELLSTART:
    doSciDwellStart(scn^.position,scn^.x,scn^.y);
SCN_DWELLEND:
    doSciDwellEnd(scn^.position,scn^.x,scn^.y);
SCN_ZOOM:
    doSciZoom;
SCN_HOTSPOTCLICK:
    doSciHotSpotClick(scn^.modifiers, scn^.position);
SCN_HOTSPOTDOUBLECLICK:
    doSciHotSpotDoubleClick(scn^.modifiers, scn^.position);
SCN_CALLTIPCLICK:
	doSciCallTipClick(scn^.position);
SCN_AUTOCSELECTION:
    doSciAutoCSelection(scn^.text);
SCN_INDICATORCLICK:
    doSciIndicatorClick(scn^.modifiers, scn^.position);
SCN_INDICATORRELEASE:
    doSciIndicatorRelease(scn^.modifiers, scn^.position);
SCN_AUTOCCANCELLED:
    doSciAutoCCancelled;
#endif
}

void TfrmJsonEditorTextView::Search(AnsiString text, const TStringSearchOptions &so)
{
	if (so.Contains(soDown))
	{
		m_se->SendEditor(SCI_CHARRIGHT);
		m_se->SendEditor(SCI_SEARCHANCHOR);
		m_se->SendEditor(SCI_SEARCHNEXT,
			so.Contains(soMatchCase)?SCFIND_MATCHCASE:0 | so.Contains(soWholeWord)?SCFIND_WHOLEWORD:0,
			(LPARAM)text.c_str());
		m_se->SendEditor(SCI_SCROLLCARET);
	}
	else
	{
		m_se->SendEditor(SCI_SEARCHANCHOR);
		m_se->SendEditor(SCI_SEARCHPREV,
			so.Contains(soMatchCase)?SCFIND_MATCHCASE:0 | so.Contains(soWholeWord)?SCFIND_WHOLEWORD:0,
			(LPARAM)text.c_str());
		m_se->SendEditor(SCI_SCROLLCARET);
	}
}

int TfrmJsonEditorTextView::SetFont(AnsiString name, int size, TFontStyles style)
{
	fontName = name;
	fontSize = size;
	m_se->setCppStyle(fontName, fontSize);
	m_se->SetExtraStyle();
	return 0;
}

void TfrmJsonEditorTextView::SetSciFocus(void)
{
	m_se->SendEditor(SCI_GRABFOCUS, 1, 0);
}



