//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "FormMain.h"
#include "FormAbout.h"
#include "FormSettings.h"
#include "LogUnit.h"
#include "Log.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TfrmMain *frmMain;
//---------------------------------------------------------------------------
__fastcall TfrmMain::TfrmMain(TComponent* Owner)
	: TForm(Owner)
{
	pcMain->ActivePage = tsParameters;
	for (int i=1; i<pcMain->PageCount; i++)
	{
    	pcMain->Pages[i]->TabVisible = false;	
	}

	// inform OS that we accepting dropping files
	DragAcceptFiles(Handle, True);	
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::FormCreate(TObject *Sender)
{
	AnsiString asConfigFile = ChangeFileExt( Application->ExeName, ".ini" );
	appSettings.Read(asConfigFile);
	if (this->BorderStyle != bsSingle)
	{
		this->Width = appSettings.frmMain.iWidth;
		this->Height = appSettings.frmMain.iHeight;
	}
	this->Top = appSettings.frmMain.iPosY;
	this->Left = appSettings.frmMain.iPosX;
	if (appSettings.frmMain.bAlwaysOnTop)
		this->FormStyle = fsStayOnTop;
	else
		this->FormStyle = fsNormal;
	if (appSettings.frmMain.bWindowMaximized)
		this->WindowState = wsMaximized;
	if (appSettings.Logging.bLogToFile)
		CLog::Instance()->SetFile(ChangeFileExt(Application->ExeName, ".log").c_str());
	else
		CLog::Instance()->SetFile("");
	cbScenarioType->ItemIndex = appSettings.Sipp.iScenarioType;
	edHost->Text = appSettings.Sipp.asRemoteHost;
	edPort->Text = appSettings.Sipp.asRemotePort;
	edScenarioFile->Text = appSettings.Sipp.asScenarioFile;
	chbInfFile->Checked = appSettings.Sipp.bUseInf;
	edInfFile->Text = appSettings.Sipp.asInfFile;
	edRate->Text = appSettings.Sipp.asRate;
	edRatePeriod->Text = appSettings.Sipp.asRatePeriod;
	edExtraParams->Text = appSettings.Sipp.asExtraParams;
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::FormCloseQuery(TObject *Sender, bool &CanClose)
{
	AnsiString asConfigFile = ChangeFileExt( Application->ExeName, ".ini" );
	appSettings.frmMain.bWindowMaximized = (this->WindowState == wsMaximized);
	if (!appSettings.frmMain.bWindowMaximized)
	{
		// these values are meaningless is wnd is maximized
		appSettings.frmMain.iWidth = this->Width;
		appSettings.frmMain.iHeight = this->Height;
		appSettings.frmMain.iPosY = this->Top;
		appSettings.frmMain.iPosX = this->Left;
	}

	appSettings.Sipp.iScenarioType = cbScenarioType->ItemIndex;
	appSettings.Sipp.asRemoteHost = edHost->Text;
	appSettings.Sipp.asRemotePort = edPort->Text;
	appSettings.Sipp.asScenarioFile = edScenarioFile->Text;
	appSettings.Sipp.bUseInf = chbInfFile->Checked;
	appSettings.Sipp.asInfFile = edInfFile->Text;
	appSettings.Sipp.asRate = edRate->Text;
	appSettings.Sipp.asRatePeriod = edRatePeriod->Text;
	appSettings.Sipp.asExtraParams = edExtraParams->Text;

	appSettings.Write(asConfigFile);

	CanClose = true;	
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::actShowAboutExecute(TObject *Sender)
{
	frmAbout->ShowModal();	
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::actShowSettingsExecute(TObject *Sender)
{
	frmSettings->appSettings = &appSettings;
	frmSettings->ShowModal();
	if (appSettings.frmMain.bAlwaysOnTop)
		this->FormStyle = fsStayOnTop;
	else
		this->FormStyle = fsNormal;
	if (appSettings.Logging.bLogToFile)
		CLog::Instance()->SetFile(ChangeFileExt(Application->ExeName, ".log").c_str());
	else
		CLog::Instance()->SetFile("");
	frmLog->SetLogLinesLimit(appSettings.Logging.iMaxUiLogLines);
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::FormShow(TObject *Sender)
{
    static bool once = false;
    if (!once)
    {
		once = true;
		frmLog->SetLogLinesLimit(appSettings.Logging.iMaxUiLogLines);		
		CLog::Instance()->SetLevel(E_LOG_TRACE);
		CLog::Instance()->callbackLog = frmLog->OnLog;
	}
	LOG("Application started");
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::FormDestroy(TObject *Sender)
{
	CLog::Instance()->Destroy();	
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::actShowLogExecute(TObject *Sender)
{
	if (!frmLog->Visible)
		frmLog->Show();
	frmLog->BringToFront();
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::cbScenarioTypeChange(TObject *Sender)
{
	bool showHost = (cbScenarioType->ItemIndex == 0);
	lblHost->Visible = showHost;
	edHost->Visible = showHost;
	lblPort->Visible = showHost;
	edPort->Visible = showHost;
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::btnLaunchClick(TObject *Sender)
{
	AnsiString cmd;
	cmd.cat_printf(" /K \"");
	cmd.cat_printf("\"%s\"", appSettings.Sipp.asExecutable.c_str());

	if (cbScenarioType->ItemIndex == 0)
	{
		cmd.cat_printf(" %s:%s", edHost->Text.c_str(), edPort->Text.c_str());
	}

	cmd.cat_printf(" -sf \"%s\"", edScenarioFile->Text.c_str());
	if (chbInfFile->Checked)
	{
		cmd.cat_printf(" -inf \"%s\"", edInfFile->Text.c_str());
	}

	cmd.cat_printf(" -m %s", this->edScenarioRunCnt->Text.c_str());
	cmd.cat_printf(" -l %s", this->edScenarioMaxSimult->Text.c_str());
	cmd.cat_printf(" -r %s", this->edRate->Text.c_str());
	cmd.cat_printf(" -rp %s", this->edRatePeriod->Text.c_str());



	cmd.cat_printf(" %s", edExtraParams->Text.c_str());
	cmd.cat_printf("\"");
	//system(cmd.c_str());
	ShellExecute(
		NULL,
		"open",
		"cmd.exe",
		cmd.c_str(), // params
		NULL,
		SW_SHOW);
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::WMDropFiles(TWMDropFiles &message)
{
    AnsiString FileName;
    char buff[MAX_PATH];

	int Count = DragQueryFile((HDROP)message.Drop, 0xFFFFFFFF, NULL, MAX_PATH);

	// index through the files and query the OS for each file name...
	for (int index = 0; index < Count; ++index)
	{
		// the following code gets the FileName of the dropped file. it
		// looks cryptic but that's only because it is. Hey, Why do you think
		// Delphi and C++Builder are so popular anyway? Look up DragQueryFile
		// the Win32.hlp Windows API help file.
		DragQueryFile((HDROP)message.Drop, index, buff, sizeof(buff));
		FileName = buff;
		AddSourceFile(FileName);
	}

    // tell the OS that you're finished...
    DragFinish((HDROP) message.Drop);
}

void TfrmMain::AddSourceFile(AnsiString name)
{
	AnsiString ext = LowerCase(ExtractFileExt(name));
	if (ext == ".xml")
	{
		edScenarioFile->Text = name;
	}
	else if (ext == ".csv")
	{
    	edInfFile->Text = name;
	}
}

