//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include <IniFiles.hpp>
#include "Settings.h"

//---------------------------------------------------------------------------

#pragma package(smart_init)

bool Settings::Read(AnsiString asFileName)
{
	TIniFile *ini = NULL;
	try
	{
		ini = new TIniFile(asFileName);
		// main window startup position
		int maxX = GetSystemMetrics(SM_CXSCREEN);
		int maxY = GetSystemMetrics(SM_CYSCREEN);
		frmMain.iWidth = ini->ReadInteger("frmMain", "AppWidth", 350);
		frmMain.iHeight = ini->ReadInteger("frmMain", "AppHeight", 300);
		if (frmMain.iWidth < 250 || frmMain.iWidth > maxX + 20)
		{
			frmMain.iWidth = 250;
		}
		if (frmMain.iHeight < 200 || frmMain.iHeight > maxY + 20)
		{
			frmMain.iHeight = 200;
		}
		frmMain.iPosX = ini->ReadInteger("frmMain", "AppPositionX", 30);
		frmMain.iPosY = ini->ReadInteger("frmMain", "AppPositionY", 30);
		if (frmMain.iPosX < 0)
			frmMain.iPosX = 0;
		if (frmMain.iPosY < 0)
			frmMain.iPosY = 0;
		if (frmMain.iPosX + frmMain.iWidth > maxX)
			frmMain.iPosX = maxX - frmMain.iWidth;
		if (frmMain.iPosY + frmMain.iHeight > maxY)
			frmMain.iPosY = maxY - frmMain.iHeight;
		frmMain.bWindowMaximized = ini->ReadBool("frmMain", "Maximized", false);
		frmMain.bAlwaysOnTop = ini->ReadBool("frmMain", "AlwaysOnTop", false);

		Logging.bLogToFile = ini->ReadBool("Logging", "LogToFile", false);
		Logging.iMaxUiLogLines = ini->ReadInteger("Logging", "MaxUiLogLines", 1000);

		AnsiString asDefaultExecutable;
		asDefaultExecutable.sprintf("%s\\sipp-win32.exe", ExtractFileDir(Application->ExeName).c_str());
		Sipp.asExecutable = ini->ReadString("SIPp", "Executable", asDefaultExecutable);
		Sipp.iScenarioType = ini->ReadInteger("SIPp", "ScenarioType", 0);
		Sipp.asRemoteHost = ini->ReadString("SIPp", "RemoteHost", "127.0.0.1");
		Sipp.asRemotePort = ini->ReadString("SIPp", "RemotePort", "5060");
		Sipp.bUseInf = ini->ReadBool("SIPp", "UseInf", false);
		Sipp.asInfFile = ini->ReadString("SIPp", "InfFile", "");
		Sipp.asScenarioRunCnt = ini->ReadString("SIPp", "ScenarioRunCnt", "1");
		Sipp.asScenarioMaxSimult = ini->ReadString("SIPp", "ScenarioMaxSimult", "1");
		Sipp.asRate = ini->ReadString("SIPp", "Rate", "10");
		Sipp.asRatePeriod = ini->ReadString("SIPp", "RatePeriod", "10000");
		Sipp.asExtraParams = ini->ReadString("SIPp", "ExtraParams", "");
		Sipp.asScenarioFile = ini->ReadString("SIPp", "ScenarioFile", "");


		delete ini;
	}
	catch (...)
	{
		if (ini)
		{
			delete ini;
		}
		return false;
	}
	return true;
}

bool Settings::Write(AnsiString asFileName)
{
	TIniFile *ini = NULL;
	try
	{
		ini = new TIniFile(asFileName);
		ini->WriteInteger("frmMain", "AppWidth", frmMain.iWidth);
		ini->WriteInteger("frmMain", "AppHeight", frmMain.iHeight);
		ini->WriteInteger("frmMain", "AppPositionX", frmMain.iPosX);
		ini->WriteInteger("frmMain", "AppPositionY", frmMain.iPosY);
		ini->WriteBool("frmMain", "Maximized", frmMain.bWindowMaximized);
		ini->WriteBool("frmMain", "AlwaysOnTop", frmMain.bAlwaysOnTop);

		ini->WriteBool("Logging", "LogToFile", Logging.bLogToFile);
		ini->WriteInteger("Logging", "MaxUiLogLines", Logging.iMaxUiLogLines);

		ini->WriteString("SIPp", "Executable", Sipp.asExecutable);
		ini->WriteInteger("SIPp", "ScenarioType", Sipp.iScenarioType);
		ini->WriteString("SIPp", "RemoteHost", Sipp.asRemoteHost);
		ini->WriteString("SIPp", "RemotePort", Sipp.asRemotePort);
		ini->WriteString("SIPp", "ScenarioFile", Sipp.asScenarioFile);
		ini->WriteBool("SIPp", "UseInf", Sipp.bUseInf);
		ini->WriteString("SIPp", "InfFile", Sipp.asInfFile);
		ini->WriteString("SIPp", "ScenarioRunCnt", Sipp.asScenarioRunCnt);
		ini->WriteString("SIPp", "ScenarioMaxSimult", Sipp.asScenarioMaxSimult);
		ini->WriteString("SIPp", "Rate", Sipp.asRate);
		ini->WriteString("SIPp", "RatePeriod", Sipp.asRatePeriod);
		ini->WriteString("SIPp", "ExtraParams", Sipp.asExtraParams);

		delete ini;
	}
	catch (...)
	{
		if (ini)
		{
			delete ini;
		}
		return false;
	}
	return true;

}
